//------------------------------------------------------------------------------
//
//  npctrl.c
//  npe
//
//  Created by Shane Saxon on 2/19/10.
//  Copyright Saxon Digital 2010. All rights reserved.
//
//------------------------------------------------------------------------------

#include "npdata.h"
#include "npio.h"
#include "npctrl.h"

#include "io/npglut.h"
#include "ctrl/npengine.h"

void npCtrlVector (int command, void* dataRef);


//------------------------------------------------------------------------------
void npInitCtrl (void* dataRef)
{
	npInitEngine (dataRef);
}

//------------------------------------------------------------------------------
void npCloseCtrl (void* dataRef)
{
	npCloseEngine (dataRef);
}

//------------------------------------------------------------------------------
int npMainAppLoop (void* dataRef)
{
	int err = 0;

#ifdef _LINUX_
//	MainAppLoopLinux();
#endif

#ifdef _MSW_
//	MainAppLoopMSW();
	npInitGlut (0, 0, dataRef);
	npMainAppLoopGlut (dataRef);
#endif

#ifdef _OSX_
	//	MainAppLoopOSX();
#endif


	return err;
}


//------------------------------------------------------------------------------
void npCtrlCommand (int command, void* dataRef)
{
	pData data = (pData) dataRef;

	NPplotPtr plot = data->map.plot[data->map.plotIndex];

	switch (command)
	{
		case kFullScreenExit : npGlutFullScreenExit(); break;

		//translate the selected objects relative to the camera
		case kGraphXdecrease : npCtrlVector(command, dataRef); break;
		case kGraphXdecreaseOff : npCtrlVector(command, dataRef); break;
		case kGraphXincrease : npCtrlVector(command, dataRef); break;
		case kGraphXincreaseOff : npCtrlVector(command, dataRef); break;

		case kGraphYdecrease : npCtrlVector(command, dataRef); break;
		case kGraphYdecreaseOff : npCtrlVector(command, dataRef); break;
		case kGraphYincrease : npCtrlVector(command, dataRef); break;
		case kGraphYincreaseOff : npCtrlVector(command, dataRef); break;

		case kGraphZdecrease : npCtrlVector(command, dataRef); break;
		case kGraphZdecreaseOff : npCtrlVector(command, dataRef); break;
		case kGraphZincrease : npCtrlVector(command, dataRef); break;
		case kGraphZincreaseOff : npCtrlVector(command, dataRef); break;

		case kGraphRotateLeft : npCtrlVector(command, dataRef); break;
		case kGraphRotateLeftOff : npCtrlVector(command, dataRef); break;
		case kGraphRotateRight : npCtrlVector(command, dataRef); break;
		case kGraphRotateRightOff : npCtrlVector(command, dataRef); break;

		case kGraphRotateUp : npCtrlVector(command, dataRef); break;
		case kGraphRotateUpOff : npCtrlVector(command, dataRef); break;
		case kGraphRotateDown : npCtrlVector(command, dataRef); break;
		case kGraphRotateDownOff : npCtrlVector(command, dataRef); break;

		case kGraphRotateCCW : npCtrlVector(command, dataRef); break;
		case kGraphRotateCCWOff : npCtrlVector(command, dataRef); break;
		case kGraphRotateCW : npCtrlVector(command, dataRef); break;
		case kGraphRotateCWOff : npCtrlVector(command, dataRef); break;

		default : break;
	}
}


// update the translateVect for selected objects
// translateVect is relative to global coordinates
// translateRate is relative to the camera
//------------------------------------------------------------------------------
void npCtrlVector (int command, void* dataRef)
{
	pData data = (pData) dataRef;

	NPplotPtr plot = data->map.plot[data->map.plotIndex];

	switch (command)
	{
		case kGraphXdecrease : plot->translateRate.x = -0.1f; break;
		case kGraphXdecreaseOff : 
			if (plot->translateRate.x < 0.0f)	//graceful multi-key behavior
				plot->translateRate.x = 0.0f; 
			break;
		case kGraphXincrease : plot->translateRate.x = 0.1f; break;
		case kGraphXincreaseOff : 
			if (plot->translateRate.x > 0.0f)
				plot->translateRate.x = 0.0f; 
			break;

		case kGraphYdecrease : plot->translateRate.y = -0.1f; break;
		case kGraphYdecreaseOff : 
			if (plot->translateRate.y < 0.0f)
				plot->translateRate.y = 0.0f;
			break;
		case kGraphYincrease : plot->translateRate.y = 0.1f; break;
		case kGraphYincreaseOff : 
			if (plot->translateRate.y > 0.0f)
				plot->translateRate.y = 0.0f;
			break;

		case kGraphZdecrease : plot->translateRate.z = -0.1f; break;
		case kGraphZdecreaseOff : 
			if (plot->translateRate.z < 0.0f)
				plot->translateRate.z = 0.0f;
			break;
		case kGraphZincrease : plot->translateRate.z = 0.1f; break;
		case kGraphZincreaseOff : 
			if (plot->translateRate.z > 0.0f)
				plot->translateRate.z = 0.0f;
			break;
		
		case kGraphRotateLeft : plot->rotateRate.y = -0.005f; break;
		case kGraphRotateLeftOff : 
			if (plot->rotateRate.y < 0.0f)
				plot->rotateRate.y = 0.0f;
			break;
		case kGraphRotateRight : plot->rotateRate.y = 0.005f; break;
		case kGraphRotateRightOff : 
			if (plot->rotateRate.y > 0.0f)
				plot->rotateRate.y = 0.0f;
			break;

		case kGraphRotateUp : plot->rotateRate.x = -0.005f; break;
		case kGraphRotateUpOff : 
			if (plot->rotateRate.x < 0.0f)
				plot->rotateRate.x = 0.0f;
			break;
		case kGraphRotateDown : plot->rotateRate.x = 0.005f; break;
		case kGraphRotateDownOff : 
			if (plot->rotateRate.x > 0.0f)
				plot->rotateRate.x = 0.0f;
			break;

		case kGraphRotateCCW : plot->rotateRate.z = -0.005f; break;
		case kGraphRotateCCWOff : 
			if (plot->rotateRate.z < 0.0f)
				plot->rotateRate.z = 0.0f;
			break;
		case kGraphRotateCW : plot->rotateRate.z = 0.005f; break;
		case kGraphRotateCWOff : 
			if (plot->rotateRate.z > 0.0f)
				plot->rotateRate.z = 0.0f;
			break;

		default : break;
	}
}


//------------------------------------------------------------------------------
void npCtrlUpdateData (void* dataRef)
{
	npDataUpdate (dataRef);
}


//------------------------------------------------------------------------------
void npCtrlUpdateEngine (void* dataRef)
{
	npEngineUpdate (dataRef);
}


